@@ -35,26 +35,206 @@
  * Author: Shannon McPherson <shannon@lunarg.com>
  */
 
-// Allow use of STL min and max functions in Windows
-#define NOMINMAX
-
 #include "chassis.h"
 #include "core_validation.h"
 
+// This LUT is created to allow a static listing of each VUID that is covered by drawdispatch commands
+// clang-format off
+static const std::map<CMD_TYPE, DrawDispatchVuid> drawdispatch_vuid = {
+    {CMD_DRAW, {
+        "VUID-vkCmdDraw-commandBuffer-cmdpool",
+        "VUID-vkCmdDraw-renderpass",
+        "VUID-vkCmdDraw-None-02700",
+        "VUID-vkCmdDraw-commandBuffer-02701",
+        "VUID-vkCmdDraw-None-04007",
+        "VUID-vkCmdDraw-None-04008",
+        "VUID-vkCmdDraw-None-02697",
+        "VUID-vkCmdDraw-renderPass-02684",
+        "VUID-vkCmdDraw-subpass-02685",
+        "VUID-vkCmdDraw-sampleLocationsEnable-02689"
+    }},
+    {CMD_DRAWINDEXED, {
+        "VUID-vkCmdDrawIndexed-commandBuffer-cmdpool",
+        "VUID-vkCmdDrawIndexed-renderpass",
+        "VUID-vkCmdDrawIndexed-None-02700",
+        "VUID-vkCmdDrawIndexed-commandBuffer-02701",
+        "VUID-vkCmdDrawIndexed-None-04007",
+        "VUID-vkCmdDrawIndexed-None-04008",
+        "VUID-vkCmdDrawIndexed-None-02697",
+        "VUID-vkCmdDrawIndexed-renderPass-02684",
+        "VUID-vkCmdDrawIndexed-subpass-02685",
+        "VUID-vkCmdDrawIndexed-sampleLocationsEnable-02689"
+    }},
+    {CMD_DRAWINDIRECT, {
+        "VUID-vkCmdDrawIndirect-commandBuffer-cmdpool",
+        "VUID-vkCmdDrawIndirect-renderpass",
+        "VUID-vkCmdDrawIndirect-None-02700",
+        "VUID-vkCmdDrawIndirect-commandBuffer-02701",
+        "VUID-vkCmdDrawIndirect-None-04007",
+        "VUID-vkCmdDrawIndirect-None-04008",
+        "VUID-vkCmdDrawIndirect-None-02697",
+        "VUID-vkCmdDrawIndirect-renderPass-02684",
+        "VUID-vkCmdDrawIndirect-subpass-02685",
+        "VUID-vkCmdDrawIndirect-sampleLocationsEnable-02689"
+    }},
+    {CMD_DRAWINDEXEDINDIRECT, {
+        "VUID-vkCmdDrawIndexedIndirect-commandBuffer-cmdpool",
+        "VUID-vkCmdDrawIndexedIndirect-renderpass",
+        "VUID-vkCmdDrawIndexedIndirect-None-02700",
+        "VUID-vkCmdDrawIndexedIndirect-commandBuffer-02701",
+        "VUID-vkCmdDrawIndexedIndirect-None-04007",
+        "VUID-vkCmdDrawIndexedIndirect-None-04008",
+        "VUID-vkCmdDrawIndexedIndirect-None-02697",
+        "VUID-vkCmdDrawIndexedIndirect-renderPass-02684",
+        "VUID-vkCmdDrawIndexedIndirect-subpass-02685",
+        "VUID-vkCmdDrawIndirect-sampleLocationsEnable-02689"
+    }},
+    {CMD_DISPATCH, {
+        "VUID-vkCmdDispatch-commandBuffer-cmdpool",
+        "VUID-vkCmdDispatch-renderpass",
+        "VUID-vkCmdDispatch-None-02700",
+         kVUIDUndefined, // dynamic_state
+         kVUIDUndefined, // vertex_binding
+         kVUIDUndefined, // vertex_binding_null
+         "VUID-vkCmdDispatch-None-02697",
+         kVUIDUndefined, // render_pass_compatible
+         kVUIDUndefined, // subpass_index
+         kVUIDUndefined, // sample_location
+    }},
+    {CMD_DISPATCHINDIRECT, {
+        "VUID-vkCmdDispatchIndirect-commandBuffer-cmdpool",
+        "VUID-vkCmdDispatchIndirect-renderpass",
+        "VUID-vkCmdDispatchIndirect-None-02700",
+        kVUIDUndefined, // dynamic_state
+        kVUIDUndefined, // vertex_binding
+         kVUIDUndefined, // vertex_binding_null
+        "VUID-vkCmdDispatchIndirect-None-02697",
+        kVUIDUndefined, // render_pass_compatible
+        kVUIDUndefined, // subpass_index
+        kVUIDUndefined, // sample_location
+    }},
+    {CMD_DRAWINDIRECTCOUNT, {
+        "VUID-vkCmdDrawIndirectCount-commandBuffer-cmdpool",
+        "VUID-vkCmdDrawIndirectCount-renderpass",
+        "VUID-vkCmdDrawIndirectCount-None-02700",
+        "VUID-vkCmdDrawIndirectCount-commandBuffer-02701",
+        "VUID-vkCmdDrawIndirectCount-None-04007",
+        "VUID-vkCmdDrawIndirectCount-None-04008",
+        "VUID-vkCmdDrawIndirectCount-None-02697",
+        "VUID-vkCmdDrawIndirectCount-renderPass-02684",
+        "VUID-vkCmdDrawIndirectCount-subpass-02685",
+        "VUID-vkCmdDrawIndirectCount-sampleLocationsEnable-02689"
+    }},
+    {CMD_DRAWINDEXEDINDIRECTCOUNT,{
+        "VUID-vkCmdDrawIndexedIndirectCount-commandBuffer-cmdpool",
+        "VUID-vkCmdDrawIndexedIndirectCount-renderpass",
+        "VUID-vkCmdDrawIndexedIndirectCount-None-02700",
+        "VUID-vkCmdDrawIndexedIndirectCount-commandBuffer-02701",
+        "VUID-vkCmdDrawIndexedIndirectCount-None-04007",
+        "VUID-vkCmdDrawIndexedIndirectCount-None-04008",
+        "VUID-vkCmdDrawIndexedIndirectCount-None-02697",
+        "VUID-vkCmdDrawIndexedIndirectCount-renderPass-02684",
+        "VUID-vkCmdDrawIndexedIndirectCount-subpass-02685",
+        "VUID-vkCmdDrawIndexedIndirectCount-sampleLocationsEnable-02689"
+    }},
+    {CMD_TRACERAYSNV, {
+        "VUID-vkCmdTraceRaysNV-commandBuffer-cmdpool",
+        "VUID-vkCmdTraceRaysNV-renderpass",
+        "VUID-vkCmdTraceRaysNV-None-02700",
+        "VUID-vkCmdTraceRaysNV-commandBuffer-02701",
+        kVUIDUndefined, // vertex_binding
+        kVUIDUndefined, // vertex_binding_null
+        "VUID-vkCmdTraceRaysNV-None-02697",
+        kVUIDUndefined, // render_pass_compatible
+        kVUIDUndefined, // subpass_index
+        kVUIDUndefined, // sample_location
+    }},
+    {CMD_TRACERAYSKHR, {
+        "VUID-vkCmdTraceRaysKHR-commandBuffer-cmdpool",
+        "VUID-vkCmdTraceRaysKHR-renderpass",
+        "VUID-vkCmdTraceRaysKHR-None-02700",
+        "VUID-vkCmdTraceRaysKHR-commandBuffer-02701",
+        kVUIDUndefined, // vertex_binding
+        kVUIDUndefined, // vertex_binding_null
+        "VUID-vkCmdTraceRaysKHR-None-02697",
+        kVUIDUndefined, // render_pass_compatible
+        kVUIDUndefined, // subpass_index
+        kVUIDUndefined, // sample_location
+    }},
+    {CMD_TRACERAYSINDIRECTKHR, {
+        "VUID-vkCmdTraceRaysIndirectKHR-commandBuffer-cmdpool",
+        "VUID-vkCmdTraceRaysIndirectKHR-renderpass",
+        "VUID-vkCmdTraceRaysIndirectKHR-None-02700",
+        "VUID-vkCmdTraceRaysIndirectKHR-commandBuffer-02701",
+        kVUIDUndefined, // vertex_binding
+        kVUIDUndefined, // vertex_binding_null
+        "VUID-vkCmdTraceRaysIndirectKHR-None-02697",
+        kVUIDUndefined, // render_pass_compatible
+        kVUIDUndefined, // subpass_index
+        kVUIDUndefined, // sample_location
+    }},
+    {CMD_DRAWMESHTASKSNV, {
+        "VUID-vkCmdDrawMeshTasksNV-commandBuffer-cmdpool",
+        "VUID-vkCmdDrawMeshTasksNV-renderpass",
+        "VUID-vkCmdDrawMeshTasksNV-None-02700",
+        "VUID-vkCmdDrawMeshTasksNV-commandBuffer-02701",
+        kVUIDUndefined, // vertex_binding
+        kVUIDUndefined, // vertex_binding_null
+        "VUID-vkCmdDrawMeshTasksNV-None-02697",
+        "VUID-vkCmdDrawMeshTasksNV-renderPass-02684",
+        "VUID-vkCmdDrawMeshTasksNV-subpass-02685",
+        "VUID-vkCmdDrawMeshTasksNV-sampleLocationsEnable-02689"
+    }},
+    {CMD_DRAWMESHTASKSINDIRECTNV, {
+        "VUID-vkCmdDrawMeshTasksIndirectNV-commandBuffer-cmdpool",
+        "VUID-vkCmdDrawMeshTasksIndirectNV-renderpass",
+        "VUID-vkCmdDrawMeshTasksIndirectNV-None-02700",
+        "VUID-vkCmdDrawMeshTasksIndirectNV-commandBuffer-02701",
+         kVUIDUndefined, // vertex_binding
+         kVUIDUndefined, // vertex_binding_null
+        "VUID-vkCmdDrawMeshTasksIndirectNV-None-02697",
+        "VUID-vkCmdDrawMeshTasksIndirectNV-renderPass-02684",
+        "VUID-vkCmdDrawMeshTasksIndirectNV-subpass-02685",
+        "VUID-vkCmdDrawMeshTasksIndirectNV-sampleLocationsEnable-02689"
+    }},
+    {CMD_DRAWMESHTASKSINDIRECTCOUNTNV, {
+        "VUID-vkCmdDrawMeshTasksIndirectCountNV-commandBuffer-cmdpool",
+        "VUID-vkCmdDrawMeshTasksIndirectCountNV-renderpass",
+        "VUID-vkCmdDrawMeshTasksIndirectCountNV-None-02700",
+        "VUID-vkCmdDrawMeshTasksIndirectCountNV-commandBuffer-02701",
+        kVUIDUndefined, // vertex_binding
+        kVUIDUndefined, // vertex_binding_null
+        "VUID-vkCmdDrawMeshTasksIndirectCountNV-None-02697",
+        "VUID-vkCmdDrawMeshTasksIndirectCountNV-renderPass-02684",
+        "VUID-vkCmdDrawMeshTasksIndirectCountNV-subpass-02685",
+        "VUID-vkCmdDrawMeshTasksIndirectCountNV-sampleLocationsEnable-02689"
+    }},
+    // Used if invalid cmd_type is used
+    {CMD_NONE, {kVUIDUndefined, kVUIDUndefined, kVUIDUndefined, kVUIDUndefined, kVUIDUndefined, kVUIDUndefined, kVUIDUndefined, kVUIDUndefined, kVUIDUndefined}}
+};
+// clang-format on
+
+// Getter function to provide kVUIDUndefined in case an invalid cmd_type is passed in
+const DrawDispatchVuid &CoreChecks::GetDrawDispatchVuid(CMD_TYPE cmd_type) const {
+    if (drawdispatch_vuid.find(cmd_type) != drawdispatch_vuid.cend()) {
+        return drawdispatch_vuid.at(cmd_type);
+    } else {
+        return drawdispatch_vuid.at(CMD_NONE);
+    }
+}
+
 // Generic function to handle validation for all CmdDraw* type functions
 bool CoreChecks::ValidateCmdDrawType(VkCommandBuffer cmd_buffer, bool indexed, VkPipelineBindPoint bind_point, CMD_TYPE cmd_type,
-                                     const char *caller, VkQueueFlags queue_flags, const char *queue_flag_code,
-                                     const char *renderpass_msg_code, const char *pipebound_msg_code,
-                                     const char *dynamic_state_msg_code, const char *vtx_binding_msg_code) const {
+                                     const char *caller, VkQueueFlags queue_flags) const {
     bool skip = false;
+    const DrawDispatchVuid vuid = GetDrawDispatchVuid(cmd_type);
     const CMD_BUFFER_STATE *cb_state = GetCBState(cmd_buffer);
     if (cb_state) {
-        skip |= ValidateCmdQueueFlags(cb_state, caller, queue_flags, queue_flag_code);
+        skip |= ValidateCmdQueueFlags(cb_state, caller, queue_flags, vuid.queue_flag);
         skip |= ValidateCmd(cb_state, cmd_type, caller);
-        skip |= ValidateCmdBufDrawState(cb_state, cmd_type, indexed, bind_point, caller, pipebound_msg_code, dynamic_state_msg_code,
-                                        vtx_binding_msg_code);
-        skip |= (VK_PIPELINE_BIND_POINT_GRAPHICS == bind_point) ? OutsideRenderPass(cb_state, caller, renderpass_msg_code)
-                                                                : InsideRenderPass(cb_state, caller, renderpass_msg_code);
+        skip |= ValidateCmdBufDrawState(cb_state, cmd_type, indexed, bind_point, caller);
+        skip |= (VK_PIPELINE_BIND_POINT_GRAPHICS == bind_point) ? OutsideRenderPass(cb_state, caller, vuid.inside_renderpass)
+                                                                : InsideRenderPass(cb_state, caller, vuid.inside_renderpass);
     }
     return skip;
 }
@@ -62,16 +242,13 @@ bool CoreChecks::ValidateCmdDrawType(VkCommandBuffer cmd_buffer, bool indexed, V
 bool CoreChecks::PreCallValidateCmdDraw(VkCommandBuffer commandBuffer, uint32_t vertexCount, uint32_t instanceCount,
                                         uint32_t firstVertex, uint32_t firstInstance) const {
     return ValidateCmdDrawType(commandBuffer, false, VK_PIPELINE_BIND_POINT_GRAPHICS, CMD_DRAW, "vkCmdDraw()",
-                               VK_QUEUE_GRAPHICS_BIT, "VUID-vkCmdDraw-commandBuffer-cmdpool", "VUID-vkCmdDraw-renderpass",
-                               "VUID-vkCmdDraw-None-02700", "VUID-vkCmdDraw-commandBuffer-02701", "VUID-vkCmdDraw-None-02720");
+                               VK_QUEUE_GRAPHICS_BIT);
 }
 
 bool CoreChecks::PreCallValidateCmdDrawIndexed(VkCommandBuffer commandBuffer, uint32_t indexCount, uint32_t instanceCount,
                                                uint32_t firstIndex, int32_t vertexOffset, uint32_t firstInstance) const {
     bool skip = ValidateCmdDrawType(commandBuffer, true, VK_PIPELINE_BIND_POINT_GRAPHICS, CMD_DRAWINDEXED, "vkCmdDrawIndexed()",
-                                    VK_QUEUE_GRAPHICS_BIT, "VUID-vkCmdDrawIndexed-commandBuffer-cmdpool",
-                                    "VUID-vkCmdDrawIndexed-renderpass", "VUID-vkCmdDrawIndexed-None-02700",
-                                    "VUID-vkCmdDrawIndexed-commandBuffer-02701", "VUID-vkCmdDrawIndexed-None-02720");
+                                    VK_QUEUE_GRAPHICS_BIT);
     const CMD_BUFFER_STATE *cb_state = GetCBState(commandBuffer);
     if (!skip && (cb_state->status & CBSTATUS_INDEX_BUFFER_BOUND)) {
         unsigned int index_size = 0;
@@ -85,13 +262,12 @@ bool CoreChecks::PreCallValidateCmdDrawIndexed(VkCommandBuffer commandBuffer, ui
         }
         VkDeviceSize end_offset = (index_size * ((VkDeviceSize)firstIndex + indexCount)) + index_buffer_binding.offset;
         if (end_offset > index_buffer_binding.size) {
-            skip |= log_msg(report_data, VK_DEBUG_REPORT_ERROR_BIT_EXT, VK_DEBUG_REPORT_OBJECT_TYPE_BUFFER_EXT,
-                            HandleToUint64(index_buffer_binding.buffer), "VUID-vkCmdDrawIndexed-indexSize-00463",
-                            "vkCmdDrawIndexed() index size (%d) * (firstIndex (%d) + indexCount (%d)) "
-                            "+ binding offset (%" PRIuLEAST64 ") = an ending offset of %" PRIuLEAST64
-                            " bytes, "
-                            "which is greater than the index buffer size (%" PRIuLEAST64 ").",
-                            index_size, firstIndex, indexCount, index_buffer_binding.offset, end_offset, index_buffer_binding.size);
+            skip |=
+                LogError(index_buffer_binding.buffer, "VUID-vkCmdDrawIndexed-indexSize-00463",
+                         "vkCmdDrawIndexed() index size (%d) * (firstIndex (%d) + indexCount (%d)) "
+                         "+ binding offset (%" PRIuLEAST64 ") = an ending offset of %" PRIuLEAST64
+                         " bytes, which is greater than the index buffer size (%" PRIuLEAST64 ").",
+                         index_size, firstIndex, indexCount, index_buffer_binding.offset, end_offset, index_buffer_binding.size);
         }
     }
     return skip;
@@ -100,9 +276,7 @@ bool CoreChecks::PreCallValidateCmdDrawIndexed(VkCommandBuffer commandBuffer, ui
 bool CoreChecks::PreCallValidateCmdDrawIndirect(VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset, uint32_t count,
                                                 uint32_t stride) const {
     bool skip = ValidateCmdDrawType(commandBuffer, false, VK_PIPELINE_BIND_POINT_GRAPHICS, CMD_DRAWINDIRECT, "vkCmdDrawIndirect()",
-                                    VK_QUEUE_GRAPHICS_BIT, "VUID-vkCmdDrawIndirect-commandBuffer-cmdpool",
-                                    "VUID-vkCmdDrawIndirect-renderpass", "VUID-vkCmdDrawIndirect-None-02700",
-                                    "VUID-vkCmdDrawIndirect-commandBuffer-02701", "VUID-vkCmdDrawIndirect-None-02720");
+                                    VK_QUEUE_GRAPHICS_BIT);
     const BUFFER_STATE *buffer_state = GetBufferState(buffer);
     skip |= ValidateMemoryIsBoundToBuffer(buffer_state, "vkCmdDrawIndirect()", "VUID-vkCmdDrawIndirect-buffer-02708");
     skip |= ValidateBufferUsageFlags(buffer_state, VK_BUFFER_USAGE_INDIRECT_BUFFER_BIT, true, "VUID-vkCmdDrawIndirect-buffer-02709",
@@ -121,11 +295,8 @@ bool CoreChecks::PreCallValidateCmdDrawIndirect(VkCommandBuffer commandBuffer, V
 
 bool CoreChecks::PreCallValidateCmdDrawIndexedIndirect(VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset,
                                                        uint32_t count, uint32_t stride) const {
-    bool skip = ValidateCmdDrawType(
-        commandBuffer, true, VK_PIPELINE_BIND_POINT_GRAPHICS, CMD_DRAWINDEXEDINDIRECT, "vkCmdDrawIndexedIndirect()",
-        VK_QUEUE_GRAPHICS_BIT, "VUID-vkCmdDrawIndexedIndirect-commandBuffer-cmdpool", "VUID-vkCmdDrawIndexedIndirect-renderpass",
-        "VUID-vkCmdDrawIndexedIndirect-None-02700", "VUID-vkCmdDrawIndexedIndirect-commandBuffer-02701",
-        "VUID-vkCmdDrawIndexedIndirect-None-02720");
+    bool skip = ValidateCmdDrawType(commandBuffer, true, VK_PIPELINE_BIND_POINT_GRAPHICS, CMD_DRAWINDEXEDINDIRECT,
+                                    "vkCmdDrawIndexedIndirect()", VK_QUEUE_GRAPHICS_BIT);
     const BUFFER_STATE *buffer_state = GetBufferState(buffer);
     skip |= ValidateMemoryIsBoundToBuffer(buffer_state, "vkCmdDrawIndexedIndirect()", "VUID-vkCmdDrawIndexedIndirect-buffer-02708");
     skip |= ValidateBufferUsageFlags(buffer_state, VK_BUFFER_USAGE_INDIRECT_BUFFER_BIT, true,
@@ -147,16 +318,13 @@ bool CoreChecks::PreCallValidateCmdDrawIndexedIndirect(VkCommandBuffer commandBu
 bool CoreChecks::PreCallValidateCmdDispatch(VkCommandBuffer commandBuffer, uint32_t x, uint32_t y, uint32_t z) const {
     bool skip = false;
     skip |= ValidateCmdDrawType(commandBuffer, false, VK_PIPELINE_BIND_POINT_COMPUTE, CMD_DISPATCH, "vkCmdDispatch()",
-                                VK_QUEUE_COMPUTE_BIT, "VUID-vkCmdDispatch-commandBuffer-cmdpool", "VUID-vkCmdDispatch-renderpass",
-                                "VUID-vkCmdDispatch-None-02700", kVUIDUndefined, kVUIDUndefined);
+                                VK_QUEUE_COMPUTE_BIT);
     return skip;
 }
 
 bool CoreChecks::PreCallValidateCmdDispatchIndirect(VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset) const {
     bool skip = ValidateCmdDrawType(commandBuffer, false, VK_PIPELINE_BIND_POINT_COMPUTE, CMD_DISPATCHINDIRECT,
-                                    "vkCmdDispatchIndirect()", VK_QUEUE_COMPUTE_BIT,
-                                    "VUID-vkCmdDispatchIndirect-commandBuffer-cmdpool", "VUID-vkCmdDispatchIndirect-renderpass",
-                                    "VUID-vkCmdDispatchIndirect-None-02700", kVUIDUndefined, kVUIDUndefined);
+                                    "vkCmdDispatchIndirect()", VK_QUEUE_COMPUTE_BIT);
     const BUFFER_STATE *buffer_state = GetBufferState(buffer);
     skip |= ValidateMemoryIsBoundToBuffer(buffer_state, "vkCmdDispatchIndirect()", "VUID-vkCmdDispatchIndirect-buffer-02708");
     skip |=
@@ -168,17 +336,11 @@ bool CoreChecks::ValidateCmdDrawIndirectCount(VkCommandBuffer commandBuffer, VkB
                                               VkBuffer countBuffer, VkDeviceSize countBufferOffset, uint32_t maxDrawCount,
                                               uint32_t stride, const char *apiName) const {
     bool skip = false;
-    if (offset & 3) {
-        skip |= log_msg(report_data, VK_DEBUG_REPORT_ERROR_BIT_EXT, VK_DEBUG_REPORT_OBJECT_TYPE_COMMAND_BUFFER_EXT,
-                        HandleToUint64(commandBuffer), "VUID-vkCmdDrawIndirectCount-offset-02710",
-                        "%s() parameter, VkDeviceSize offset (0x%" PRIxLEAST64 "), is not a multiple of 4.", apiName, offset);
-    }
-
-    if (countBufferOffset & 3) {
-        skip |= log_msg(report_data, VK_DEBUG_REPORT_ERROR_BIT_EXT, VK_DEBUG_REPORT_OBJECT_TYPE_COMMAND_BUFFER_EXT,
-                        HandleToUint64(commandBuffer), "VUID-vkCmdDrawIndirectCount-countBufferOffset-02716",
-                        "%s() parameter, VkDeviceSize countBufferOffset (0x%" PRIxLEAST64 "), is not a multiple of 4.", apiName,
-                        countBufferOffset);
+    if ((api_version >= VK_API_VERSION_1_2) && (enabled_features.core12.drawIndirectCount == VK_FALSE)) {
+        skip |= LogError(commandBuffer, "VUID-vkCmdDrawIndirectCount-None-02836",
+                         "%s(): Starting in Vulkan 1.2 the VkPhysicalDeviceVulkan12Features::drawIndirectCount must be enabled to "
+                         "call this command.",
+                         apiName);
     }
     skip |= ValidateCmdDrawStrideWithStruct(commandBuffer, "VUID-vkCmdDrawIndirectCount-stride-03110", stride, apiName,
                                             sizeof(VkDrawIndirectCommand));
@@ -189,9 +351,7 @@ bool CoreChecks::ValidateCmdDrawIndirectCount(VkCommandBuffer commandBuffer, VkB
     }
 
     skip |= ValidateCmdDrawType(commandBuffer, false, VK_PIPELINE_BIND_POINT_GRAPHICS, CMD_DRAWINDIRECTCOUNT, apiName,
-                                VK_QUEUE_GRAPHICS_BIT, "VUID-vkCmdDrawIndirectCount-commandBuffer-cmdpool",
-                                "VUID-vkCmdDrawIndirectCount-renderpass", "VUID-vkCmdDrawIndirectCount-None-02700",
-                                "VUID-vkCmdDrawIndirectCount-commandBuffer-02701", "VUID-vkCmdDrawIndirectCount-None-02720");
+                                VK_QUEUE_GRAPHICS_BIT);
     const BUFFER_STATE *buffer_state = GetBufferState(buffer);
     const BUFFER_STATE *count_buffer_state = GetBufferState(countBuffer);
     skip |= ValidateMemoryIsBoundToBuffer(buffer_state, apiName, "VUID-vkCmdDrawIndirectCount-buffer-02708");
@@ -208,60 +368,45 @@ bool CoreChecks::PreCallValidateCmdDrawIndirectCountKHR(VkCommandBuffer commandB
                                                         VkBuffer countBuffer, VkDeviceSize countBufferOffset, uint32_t maxDrawCount,
                                                         uint32_t stride) const {
     return ValidateCmdDrawIndirectCount(commandBuffer, buffer, offset, countBuffer, countBufferOffset, maxDrawCount, stride,
-                                        "VkCmdDrawIndirectCountKHR");
+                                        "vkCmdDrawIndirectCountKHR");
 }
 
 bool CoreChecks::PreCallValidateCmdDrawIndirectCount(VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset,
                                                      VkBuffer countBuffer, VkDeviceSize countBufferOffset, uint32_t maxDrawCount,
                                                      uint32_t stride) const {
     return ValidateCmdDrawIndirectCount(commandBuffer, buffer, offset, countBuffer, countBufferOffset, maxDrawCount, stride,
-                                        "VkCmdDrawIndirectCount");
+                                        "vkCmdDrawIndirectCount");
 }
 
 bool CoreChecks::ValidateCmdDrawIndexedIndirectCount(VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset,
                                                      VkBuffer countBuffer, VkDeviceSize countBufferOffset, uint32_t maxDrawCount,
-                                                     uint32_t stride) const {
+                                                     uint32_t stride, const char *apiName) const {
     bool skip = false;
-    if (offset & 3) {
-        skip |= log_msg(
-            report_data, VK_DEBUG_REPORT_ERROR_BIT_EXT, VK_DEBUG_REPORT_OBJECT_TYPE_COMMAND_BUFFER_EXT,
-            HandleToUint64(commandBuffer), "VUID-vkCmdDrawIndexedIndirectCount-offset-02710",
-            "vkCmdDrawIndexedIndirectCount() parameter, VkDeviceSize offset (0x%" PRIxLEAST64 "), is not a multiple of 4.", offset);
+    if ((api_version >= VK_API_VERSION_1_2) && (enabled_features.core12.drawIndirectCount == VK_FALSE)) {
+        skip |= LogError(commandBuffer, "VUID-vkCmdDrawIndexedIndirect-None-02837",
+                         "%s(): Starting in Vulkan 1.2 the VkPhysicalDeviceVulkan12Features::drawIndirectCount must be enabled to "
+                         "call this command.",
+                         apiName);
     }
-
-    if (countBufferOffset & 3) {
-        skip |= log_msg(report_data, VK_DEBUG_REPORT_ERROR_BIT_EXT, VK_DEBUG_REPORT_OBJECT_TYPE_COMMAND_BUFFER_EXT,
-                        HandleToUint64(commandBuffer), "VUID-vkCmdDrawIndexedIndirectCount-countBufferOffset-02716",
-                        "vkCmdDrawIndexedIndirectCount() parameter, VkDeviceSize countBufferOffset (0x%" PRIxLEAST64
-                        "), is not a multiple of 4.",
-                        countBufferOffset);
-    }
-
-    skip |= ValidateCmdDrawStrideWithStruct(commandBuffer, "VUID-vkCmdDrawIndexedIndirectCount-stride-03142", stride,
-                                            "VkDrawIndirectCommand", sizeof(VkDrawIndexedIndirectCommand));
+    skip |= ValidateCmdDrawStrideWithStruct(commandBuffer, "VUID-vkCmdDrawIndexedIndirectCount-stride-03142", stride, apiName,
+                                            sizeof(VkDrawIndexedIndirectCommand));
     if (maxDrawCount > 1) {
         const BUFFER_STATE *buffer_state = GetBufferState(buffer);
         skip |= ValidateCmdDrawStrideWithBuffer(commandBuffer, "VUID-vkCmdDrawIndexedIndirectCount-maxDrawCount-03143", stride,
-                                                "VkDrawIndirectCommand", sizeof(VkDrawIndexedIndirectCommand), maxDrawCount, offset,
-                                                buffer_state);
+                                                apiName, sizeof(VkDrawIndexedIndirectCommand), maxDrawCount, offset, buffer_state);
     }
 
-    skip |= ValidateCmdDrawType(
-        commandBuffer, true, VK_PIPELINE_BIND_POINT_GRAPHICS, CMD_DRAWINDEXEDINDIRECTCOUNT, "vkCmdDrawIndexedIndirectCount()",
-        VK_QUEUE_GRAPHICS_BIT, "VUID-vkCmdDrawIndexedIndirectCount-commandBuffer-cmdpool",
-        "VUID-vkCmdDrawIndexedIndirectCount-renderpass", "VUID-vkCmdDrawIndexedIndirectCount-None-02700",
-        "VUID-vkCmdDrawIndexedIndirectCount-commandBuffer-02701", "VUID-vkCmdDrawIndexedIndirectCount-None-02720");
+    skip |= ValidateCmdDrawType(commandBuffer, true, VK_PIPELINE_BIND_POINT_GRAPHICS, CMD_DRAWINDEXEDINDIRECTCOUNT, apiName,
+                                VK_QUEUE_GRAPHICS_BIT);
     const BUFFER_STATE *buffer_state = GetBufferState(buffer);
     const BUFFER_STATE *count_buffer_state = GetBufferState(countBuffer);
-    skip |= ValidateMemoryIsBoundToBuffer(buffer_state, "vkCmdDrawIndexedIndirectCount()",
-                                          "VUID-vkCmdDrawIndexedIndirectCount-buffer-02708");
-    skip |= ValidateMemoryIsBoundToBuffer(count_buffer_state, "vkCmdDrawIndexedIndirectCount()",
-                                          "VUID-vkCmdDrawIndexedIndirectCount-countBuffer-02714");
-    skip |= ValidateBufferUsageFlags(buffer_state, VK_BUFFER_USAGE_INDIRECT_BUFFER_BIT, true,
-                                     "VUID-vkCmdDrawIndexedIndirectCount-buffer-02709", "vkCmdDrawIndexedIndirectCount()",
-                                     "VK_BUFFER_USAGE_INDIRECT_BUFFER_BIT");
+    skip |= ValidateMemoryIsBoundToBuffer(buffer_state, apiName, "VUID-vkCmdDrawIndexedIndirectCount-buffer-02708");
+    skip |= ValidateMemoryIsBoundToBuffer(count_buffer_state, apiName, "VUID-vkCmdDrawIndexedIndirectCount-countBuffer-02714");
+    skip |=
+        ValidateBufferUsageFlags(buffer_state, VK_BUFFER_USAGE_INDIRECT_BUFFER_BIT, true,
+                                 "VUID-vkCmdDrawIndexedIndirectCount-buffer-02709", apiName, "VK_BUFFER_USAGE_INDIRECT_BUFFER_BIT");
     skip |= ValidateBufferUsageFlags(count_buffer_state, VK_BUFFER_USAGE_INDIRECT_BUFFER_BIT, true,
-                                     "VUID-vkCmdDrawIndexedIndirectCount-countBuffer-02715", "vkCmdDrawIndexedIndirectCount()",
+                                     "VUID-vkCmdDrawIndexedIndirectCount-countBuffer-02715", apiName,
                                      "VK_BUFFER_USAGE_INDIRECT_BUFFER_BIT");
     return skip;
 }
@@ -269,13 +414,15 @@ bool CoreChecks::ValidateCmdDrawIndexedIndirectCount(VkCommandBuffer commandBuff
 bool CoreChecks::PreCallValidateCmdDrawIndexedIndirectCountKHR(VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset,
                                                                VkBuffer countBuffer, VkDeviceSize countBufferOffset,
                                                                uint32_t maxDrawCount, uint32_t stride) const {
-    return ValidateCmdDrawIndexedIndirectCount(commandBuffer, buffer, offset, countBuffer, countBufferOffset, maxDrawCount, stride);
+    return ValidateCmdDrawIndexedIndirectCount(commandBuffer, buffer, offset, countBuffer, countBufferOffset, maxDrawCount, stride,
+                                               "vkCmdDrawIndexedIndirectCountKHR");
 }
 
 bool CoreChecks::PreCallValidateCmdDrawIndexedIndirectCount(VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset,
                                                             VkBuffer countBuffer, VkDeviceSize countBufferOffset,
                                                             uint32_t maxDrawCount, uint32_t stride) const {
-    return ValidateCmdDrawIndexedIndirectCount(commandBuffer, buffer, offset, countBuffer, countBufferOffset, maxDrawCount, stride);
+    return ValidateCmdDrawIndexedIndirectCount(commandBuffer, buffer, offset, countBuffer, countBufferOffset, maxDrawCount, stride,
+                                               "vkCmdDrawIndexedIndirectCount");
 }
 
 bool CoreChecks::PreCallValidateCmdTraceRaysNV(VkCommandBuffer commandBuffer, VkBuffer raygenShaderBindingTableBuffer,
@@ -285,10 +432,8 @@ bool CoreChecks::PreCallValidateCmdTraceRaysNV(VkCommandBuffer commandBuffer, Vk
                                                VkDeviceSize hitShaderBindingStride, VkBuffer callableShaderBindingTableBuffer,
                                                VkDeviceSize callableShaderBindingOffset, VkDeviceSize callableShaderBindingStride,
                                                uint32_t width, uint32_t height, uint32_t depth) const {
-    bool skip =
-        ValidateCmdDrawType(commandBuffer, true, VK_PIPELINE_BIND_POINT_RAY_TRACING_NV, CMD_TRACERAYSNV, "vkCmdTraceRaysNV()",
-                            VK_QUEUE_COMPUTE_BIT, "VUID-vkCmdTraceRaysNV-commandBuffer-cmdpool", "VUID-vkCmdTraceRaysNV-renderpass",
-                            "VUID-vkCmdTraceRaysNV-None-02700", "VUID-vkCmdTraceRaysNV-commandBuffer-02701", kVUIDUndefined);
+    bool skip = ValidateCmdDrawType(commandBuffer, true, VK_PIPELINE_BIND_POINT_RAY_TRACING_NV, CMD_TRACERAYSNV,
+                                    "vkCmdTraceRaysNV()", VK_QUEUE_COMPUTE_BIT);
     return skip;
 }
 
@@ -304,21 +449,62 @@ void CoreChecks::PostCallRecordCmdTraceRaysNV(VkCommandBuffer commandBuffer, VkB
     cb_state->hasTraceRaysCmd = true;
 }
 
+bool CoreChecks::PreCallValidateCmdTraceRaysKHR(VkCommandBuffer commandBuffer,
+                                                const VkStridedBufferRegionKHR *pRaygenShaderBindingTable,
+                                                const VkStridedBufferRegionKHR *pMissShaderBindingTable,
+                                                const VkStridedBufferRegionKHR *pHitShaderBindingTable,
+                                                const VkStridedBufferRegionKHR *pCallableShaderBindingTable, uint32_t width,
+                                                uint32_t height, uint32_t depth) const {
+    bool skip = ValidateCmdDrawType(commandBuffer, true, VK_PIPELINE_BIND_POINT_RAY_TRACING_KHR, CMD_TRACERAYSKHR,
+                                    "vkCmdTraceRaysKHR()", VK_QUEUE_COMPUTE_BIT);
+    return skip;
+}
+
+void CoreChecks::PostCallRecordCmdTraceRaysKHR(VkCommandBuffer commandBuffer,
+                                               const VkStridedBufferRegionKHR *pRaygenShaderBindingTable,
+                                               const VkStridedBufferRegionKHR *pMissShaderBindingTable,
+                                               const VkStridedBufferRegionKHR *pHitShaderBindingTable,
+                                               const VkStridedBufferRegionKHR *pCallableShaderBindingTable, uint32_t width,
+                                               uint32_t height, uint32_t depth) {
+    CMD_BUFFER_STATE *cb_state = GetCBState(commandBuffer);
+    UpdateStateCmdDrawDispatchType(cb_state, VK_PIPELINE_BIND_POINT_RAY_TRACING_KHR);
+    cb_state->hasTraceRaysCmd = true;
+}
+
+bool CoreChecks::PreCallValidateCmdTraceRaysIndirectKHR(VkCommandBuffer commandBuffer,
+                                                        const VkStridedBufferRegionKHR *pRaygenShaderBindingTable,
+                                                        const VkStridedBufferRegionKHR *pMissShaderBindingTable,
+                                                        const VkStridedBufferRegionKHR *pHitShaderBindingTable,
+                                                        const VkStridedBufferRegionKHR *pCallableShaderBindingTable,
+                                                        VkBuffer buffer, VkDeviceSize offset) const {
+    bool skip = ValidateCmdDrawType(commandBuffer, true, VK_PIPELINE_BIND_POINT_RAY_TRACING_KHR, CMD_TRACERAYSKHR,
+                                    "vkCmdTraceRaysIndirectKHR()", VK_QUEUE_COMPUTE_BIT);
+    return skip;
+}
+
+void CoreChecks::PostCallRecordCmdTraceRaysIndirectKHR(VkCommandBuffer commandBuffer,
+                                                       const VkStridedBufferRegionKHR *pRaygenShaderBindingTable,
+                                                       const VkStridedBufferRegionKHR *pMissShaderBindingTable,
+                                                       const VkStridedBufferRegionKHR *pHitShaderBindingTable,
+                                                       const VkStridedBufferRegionKHR *pCallableShaderBindingTable, VkBuffer buffer,
+                                                       VkDeviceSize offset) {
+    CMD_BUFFER_STATE *cb_state = GetCBState(commandBuffer);
+    BUFFER_STATE *buffer_state = GetBufferState(buffer);
+    UpdateStateCmdDrawDispatchType(cb_state, VK_PIPELINE_BIND_POINT_RAY_TRACING_KHR);
+    cb_state->hasTraceRaysCmd = true;
+    AddCommandBufferBindingBuffer(cb_state, buffer_state);
+}
+
 bool CoreChecks::PreCallValidateCmdDrawMeshTasksNV(VkCommandBuffer commandBuffer, uint32_t taskCount, uint32_t firstTask) const {
-    bool skip = ValidateCmdDrawType(
-        commandBuffer, false, VK_PIPELINE_BIND_POINT_GRAPHICS, CMD_DRAWMESHTASKSNV, "vkCmdDrawMeshTasksNV()", VK_QUEUE_GRAPHICS_BIT,
-        "VUID-vkCmdDrawMeshTasksNV-commandBuffer-cmdpool", "VUID-vkCmdDrawMeshTasksNV-renderpass",
-        "VUID-vkCmdDrawMeshTasksNV-None-02700", "VUID-vkCmdDrawMeshTasksNV-commandBuffer-02701", kVUIDUndefined);
+    bool skip = ValidateCmdDrawType(commandBuffer, false, VK_PIPELINE_BIND_POINT_GRAPHICS, CMD_DRAWMESHTASKSNV,
+                                    "vkCmdDrawMeshTasksNV()", VK_QUEUE_GRAPHICS_BIT);
     return skip;
 }
 
 bool CoreChecks::PreCallValidateCmdDrawMeshTasksIndirectNV(VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset,
                                                            uint32_t drawCount, uint32_t stride) const {
     bool skip = ValidateCmdDrawType(commandBuffer, false, VK_PIPELINE_BIND_POINT_GRAPHICS, CMD_DRAWMESHTASKSINDIRECTNV,
-                                    "vkCmdDrawMeshTasksIndirectNV()", VK_QUEUE_GRAPHICS_BIT,
-                                    "VUID-vkCmdDrawMeshTasksIndirectNV-commandBuffer-cmdpool",
-                                    "VUID-vkCmdDrawMeshTasksIndirectNV-renderpass", "VUID-vkCmdDrawMeshTasksIndirectNV-None-02700",
-                                    "VUID-vkCmdDrawMeshTasksIndirectNV-commandBuffer-02701", kVUIDUndefined);
+                                    "vkCmdDrawMeshTasksIndirectNV()", VK_QUEUE_GRAPHICS_BIT);
     const BUFFER_STATE *buffer_state = GetBufferState(buffer);
     skip |= ValidateMemoryIsBoundToBuffer(buffer_state, "vkCmdDrawMeshTasksIndirectNV()",
                                           "VUID-vkCmdDrawMeshTasksIndirectNV-buffer-02708");
@@ -337,11 +523,7 @@ bool CoreChecks::PreCallValidateCmdDrawMeshTasksIndirectCountNV(VkCommandBuffer
                                                                 VkBuffer countBuffer, VkDeviceSize countBufferOffset,
                                                                 uint32_t maxDrawCount, uint32_t stride) const {
     bool skip = ValidateCmdDrawType(commandBuffer, false, VK_PIPELINE_BIND_POINT_GRAPHICS, CMD_DRAWMESHTASKSINDIRECTCOUNTNV,
-                                    "vkCmdDrawMeshTasksIndirectCountNV()", VK_QUEUE_GRAPHICS_BIT,
-                                    "VUID-vkCmdDrawMeshTasksIndirectCountNV-commandBuffer-cmdpool",
-                                    "VUID-vkCmdDrawMeshTasksIndirectCountNV-renderpass",
-                                    "VUID-vkCmdDrawMeshTasksIndirectCountNV-None-02700",
-                                    "VUID-vkCmdDrawMeshTasksIndirectCountNV-commandBuffer-02701", kVUIDUndefined);
+                                    "vkCmdDrawMeshTasksIndirectCountNV()", VK_QUEUE_GRAPHICS_BIT);
     const BUFFER_STATE *buffer_state = GetBufferState(buffer);
     const BUFFER_STATE *count_buffer_state = GetBufferState(countBuffer);
     skip |= ValidateMemoryIsBoundToBuffer(buffer_state, "vkCmdDrawMeshTasksIndirectCountNV()",
@@ -349,11 +531,11 @@ bool CoreChecks::PreCallValidateCmdDrawMeshTasksIndirectCountNV(VkCommandBuffer
     skip |= ValidateMemoryIsBoundToBuffer(count_buffer_state, "vkCmdDrawMeshTasksIndirectCountNV()",
                                           "VUID-vkCmdDrawMeshTasksIndirectCountNV-countBuffer-02714");
     skip |= ValidateBufferUsageFlags(buffer_state, VK_BUFFER_USAGE_INDIRECT_BUFFER_BIT, true,
-                                     "VUID-vkCmdDrawMeshTasksIndirectCountNV-buffer-02709", "vkCmdDrawIndexedIndirectCount()",
+                                     "VUID-vkCmdDrawMeshTasksIndirectCountNV-buffer-02709", "vkCmdDrawMeshTasksIndirectCountNV()",
                                      "VK_BUFFER_USAGE_INDIRECT_BUFFER_BIT");
     skip |= ValidateBufferUsageFlags(count_buffer_state, VK_BUFFER_USAGE_INDIRECT_BUFFER_BIT, true,
-                                     "VUID-vkCmdDrawMeshTasksIndirectCountNV-countBuffer-02715", "vkCmdDrawIndexedIndirectCount()",
-                                     "VK_BUFFER_USAGE_INDIRECT_BUFFER_BIT");
+                                     "VUID-vkCmdDrawMeshTasksIndirectCountNV-countBuffer-02715",
+                                     "vkCmdDrawMeshTasksIndirectCountNV()", "VK_BUFFER_USAGE_INDIRECT_BUFFER_BIT");
     skip |= ValidateCmdDrawStrideWithStruct(commandBuffer, "VUID-vkCmdDrawMeshTasksIndirectCountNV-stride-02182", stride,
                                             "VkDrawMeshTasksIndirectCommandNV", sizeof(VkDrawMeshTasksIndirectCommandNV));
     if (maxDrawCount > 1) {
