@@ -1,6 +1,6 @@
-/* Copyright (c) 2018-2019 The Khronos Group Inc.
- * Copyright (c) 2018-2019 Valve Corporation
- * Copyright (c) 2018-2019 LunarG, Inc.
+/* Copyright (c) 2018-2020 The Khronos Group Inc.
+ * Copyright (c) 2018-2020 Valve Corporation
+ * Copyright (c) 2018-2020 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -23,6 +23,7 @@
 #include "chassis.h"
 #include "state_tracker.h"
 #include "vk_mem_alloc.h"
+#include "gpu_utils.h"
 class GpuAssisted;
 
 struct GpuAssistedDeviceMemoryBlock {
@@ -49,33 +50,6 @@ struct GpuAssistedBufferInfo {
           pipeline_bind_point(pipeline_bind_point){};
 };
 
-struct GpuAssistedQueueBarrierCommandInfo {
-    VkCommandPool barrier_command_pool = VK_NULL_HANDLE;
-    VkCommandBuffer barrier_command_buffer = VK_NULL_HANDLE;
-};
-
-// Class to encapsulate Descriptor Set allocation.  This manager creates and destroys Descriptor Pools
-// as needed to satisfy requests for descriptor sets.
-class GpuAssistedDescriptorSetManager {
-  public:
-    GpuAssistedDescriptorSetManager(GpuAssisted* dev_data);
-    ~GpuAssistedDescriptorSetManager();
-
-    VkResult GetDescriptorSet(VkDescriptorPool* desc_pool, VkDescriptorSet* desc_sets);
-    VkResult GetDescriptorSets(uint32_t count, VkDescriptorPool* pool, std::vector<VkDescriptorSet>* desc_sets);
-    void PutBackDescriptorSet(VkDescriptorPool desc_pool, VkDescriptorSet desc_set);
-
-  private:
-    static const uint32_t kItemsPerChunk = 512;
-    struct PoolTracker {
-        uint32_t size;
-        uint32_t used;
-    };
-
-    GpuAssisted* dev_data_;
-    std::unordered_map<VkDescriptorPool, struct PoolTracker> desc_pool_map_;
-};
-
 struct GpuAssistedShaderTracker {
     VkPipeline pipeline;
     VkShaderModule shader_module;
@@ -110,21 +84,30 @@ struct GpuAssistedAccelerationStructureBuildValidationState {
 };
 
 class GpuAssisted : public ValidationStateTracker {
-    bool aborted = false;
+    VkPhysicalDeviceFeatures supported_features;
     VkBool32 shaderInt64;
-    uint32_t adjusted_max_desc_sets;
-    uint32_t desc_set_bind_index;
     uint32_t unique_shader_module_id = 0;
-    std::unordered_map<uint32_t, GpuAssistedShaderTracker> shader_map;
-    std::unique_ptr<GpuAssistedDescriptorSetManager> desc_set_manager;
-    std::map<VkQueue, GpuAssistedQueueBarrierCommandInfo> queue_barrier_command_infos;
     std::unordered_map<VkCommandBuffer, std::vector<GpuAssistedBufferInfo>> command_buffer_map;  // gpu_buffer_list;
     uint32_t output_buffer_size;
-    VmaAllocator vmaAllocator = {};
-    PFN_vkSetDeviceLoaderData vkSetDeviceLoaderData;
     std::map<VkDeviceAddress, VkDeviceSize> buffer_map;
     GpuAssistedAccelerationStructureBuildValidationState acceleration_structure_validation_state;
-    std::vector<GpuAssistedBufferInfo>& GetGpuAssistedBufferInfo(const VkCommandBuffer command_buffer) {
+
+  public:
+    GpuAssisted() { container_type = LayerObjectTypeGpuAssisted; }
+
+    bool aborted = false;
+    VkDevice device;
+    VkPhysicalDevice physicalDevice;
+    uint32_t adjusted_max_desc_sets;
+    uint32_t desc_set_bind_index;
+    VkDescriptorSetLayout debug_desc_layout = VK_NULL_HANDLE;
+    VkDescriptorSetLayout dummy_desc_layout = VK_NULL_HANDLE;
+    std::unique_ptr<UtilDescriptorSetManager> desc_set_manager;
+    std::unordered_map<uint32_t, GpuAssistedShaderTracker> shader_map;
+    PFN_vkSetDeviceLoaderData vkSetDeviceLoaderData;
+    VmaAllocator vmaAllocator = {};
+    std::map<VkQueue, UtilQueueBarrierCommandInfo> queue_barrier_command_infos;
+    std::vector<GpuAssistedBufferInfo>& GetBufferInfo(const VkCommandBuffer command_buffer) {
         auto buffer_list = command_buffer_map.find(command_buffer);
         if (buffer_list == command_buffer_map.end()) {
             std::vector<GpuAssistedBufferInfo> new_list{};
@@ -133,17 +116,16 @@ class GpuAssisted : public ValidationStateTracker {
         }
         return buffer_list->second;
     }
-    void ReportSetupProblem(VkDebugReportObjectTypeEXT object_type, uint64_t object_handle,
-                            const char* const specific_message) const;
 
   public:
-    VkDescriptorSetLayout debug_desc_layout;
-    VkDescriptorSetLayout dummy_desc_layout;
+    template <typename T>
+    void ReportSetupProblem(T object, const char* const specific_message) const;
     void PreCallRecordCreateDevice(VkPhysicalDevice gpu, const VkDeviceCreateInfo* pCreateInfo,
                                    const VkAllocationCallbacks* pAllocator, VkDevice* pDevice,
                                    safe_VkDeviceCreateInfo* modified_create_info);
     void PostCallRecordCreateDevice(VkPhysicalDevice gpu, const VkDeviceCreateInfo* pCreateInfo,
                                     const VkAllocationCallbacks* pAllocator, VkDevice* pDevice, VkResult result);
+    void PostCallRecordGetBufferDeviceAddress(VkDevice device, const VkBufferDeviceAddressInfoEXT* pInfo, VkDeviceAddress address);
     void PostCallRecordGetBufferDeviceAddressKHR(VkDevice device, const VkBufferDeviceAddressInfoEXT* pInfo,
                                                  VkDeviceAddress address);
     void PostCallRecordGetBufferDeviceAddressEXT(VkDevice device, const VkBufferDeviceAddressInfoEXT* pInfo,
@@ -186,15 +168,10 @@ class GpuAssisted : public ValidationStateTracker {
                                                   const VkRayTracingPipelineCreateInfoNV* pCreateInfos,
                                                   const VkAllocationCallbacks* pAllocator, VkPipeline* pPipelines,
                                                   void* crtpl_state_data);
-    template <typename CreateInfo, typename SafeCreateInfo>
-    void PreCallRecordPipelineCreations(uint32_t count, const CreateInfo* pCreateInfos, const VkAllocationCallbacks* pAllocator,
-                                        VkPipeline* pPipelines, std::vector<std::shared_ptr<PIPELINE_STATE>>& pipe_state,
-                                        std::vector<SafeCreateInfo>* new_pipeline_create_infos,
-                                        const VkPipelineBindPoint bind_point);
-    template <typename CreateInfo>
-    void PostCallRecordPipelineCreations(const uint32_t count, const CreateInfo* pCreateInfos,
-                                         const VkAllocationCallbacks* pAllocator, VkPipeline* pPipelines,
-                                         const VkPipelineBindPoint bind_point);
+    void PreCallRecordCreateRayTracingPipelinesKHR(VkDevice device, VkPipelineCache pipelineCache, uint32_t count,
+                                                   const VkRayTracingPipelineCreateInfoKHR* pCreateInfos,
+                                                   const VkAllocationCallbacks* pAllocator, VkPipeline* pPipelines,
+                                                   void* crtpl_state_data);
     void PostCallRecordCreateGraphicsPipelines(VkDevice device, VkPipelineCache pipelineCache, uint32_t count,
                                                const VkGraphicsPipelineCreateInfo* pCreateInfos,
                                                const VkAllocationCallbacks* pAllocator, VkPipeline* pPipelines, VkResult result,
@@ -207,17 +184,19 @@ class GpuAssisted : public ValidationStateTracker {
                                                    const VkRayTracingPipelineCreateInfoNV* pCreateInfos,
                                                    const VkAllocationCallbacks* pAllocator, VkPipeline* pPipelines, VkResult result,
                                                    void* crtpl_state_data);
+    void PostCallRecordCreateRayTracingPipelinesKHR(VkDevice device, VkPipelineCache pipelineCache, uint32_t count,
+                                                    const VkRayTracingPipelineCreateInfoKHR* pCreateInfos,
+                                                    const VkAllocationCallbacks* pAllocator, VkPipeline* pPipelines,
+                                                    VkResult result, void* crtpl_state_data);
     void PreCallRecordDestroyPipeline(VkDevice device, VkPipeline pipeline, const VkAllocationCallbacks* pAllocator);
     bool InstrumentShader(const VkShaderModuleCreateInfo* pCreateInfo, std::vector<unsigned int>& new_pgm,
                           uint32_t* unique_shader_id);
     void PreCallRecordCreateShaderModule(VkDevice device, const VkShaderModuleCreateInfo* pCreateInfo,
                                          const VkAllocationCallbacks* pAllocator, VkShaderModule* pShaderModule,
                                          void* csm_state_data);
-    void AnalyzeAndReportError(CMD_BUFFER_STATE* cb_node, VkQueue queue, VkPipelineBindPoint pipeline_bind_point,
-                               uint32_t operation_index, uint32_t* const debug_output_buffer);
-    void ProcessInstrumentationBuffer(VkQueue queue, CMD_BUFFER_STATE* cb_node);
+    void AnalyzeAndGenerateMessages(VkCommandBuffer command_buffer, VkQueue queue, VkPipelineBindPoint pipeline_bind_point,
+                                    uint32_t operation_index, uint32_t* const debug_output_buffer);
     void UpdateInstrumentationBuffer(CMD_BUFFER_STATE* cb_node);
-    void SubmitBarrier(VkQueue queue);
     void PreCallRecordQueueSubmit(VkQueue queue, uint32_t submitCount, const VkSubmitInfo* pSubmits, VkFence fence);
     void PostCallRecordQueueSubmit(VkQueue queue, uint32_t submitCount, const VkSubmitInfo* pSubmits, VkFence fence,
                                    VkResult result);
@@ -245,10 +224,31 @@ class GpuAssisted : public ValidationStateTracker {
                                       VkDeviceSize hitShaderBindingStride, VkBuffer callableShaderBindingTableBuffer,
                                       VkDeviceSize callableShaderBindingOffset, VkDeviceSize callableShaderBindingStride,
                                       uint32_t width, uint32_t height, uint32_t depth);
+    void PreCallRecordCmdTraceRaysKHR(VkCommandBuffer commandBuffer, const VkStridedBufferRegionKHR* pRaygenShaderBindingTable,
+                                      const VkStridedBufferRegionKHR* pMissShaderBindingTable,
+                                      const VkStridedBufferRegionKHR* pHitShaderBindingTable,
+                                      const VkStridedBufferRegionKHR* pCallableShaderBindingTable, uint32_t width, uint32_t height,
+                                      uint32_t depth);
+    void PostCallRecordCmdTraceRaysKHR(VkCommandBuffer commandBuffer, const VkStridedBufferRegionKHR* pRaygenShaderBindingTable,
+                                       const VkStridedBufferRegionKHR* pMissShaderBindingTable,
+                                       const VkStridedBufferRegionKHR* pHitShaderBindingTable,
+                                       const VkStridedBufferRegionKHR* pCallableShaderBindingTable, uint32_t width, uint32_t height,
+                                       uint32_t depth);
+    void PreCallRecordCmdTraceRaysIndirectKHR(VkCommandBuffer commandBuffer,
+                                              const VkStridedBufferRegionKHR* pRaygenShaderBindingTable,
+                                              const VkStridedBufferRegionKHR* pMissShaderBindingTable,
+                                              const VkStridedBufferRegionKHR* pHitShaderBindingTable,
+                                              const VkStridedBufferRegionKHR* pCallableShaderBindingTable, VkBuffer buffer,
+                                              VkDeviceSize offset);
+    void PostCallRecordCmdTraceRaysIndirectKHR(VkCommandBuffer commandBuffer,
+                                               const VkStridedBufferRegionKHR* pRaygenShaderBindingTable,
+                                               const VkStridedBufferRegionKHR* pMissShaderBindingTable,
+                                               const VkStridedBufferRegionKHR* pHitShaderBindingTable,
+                                               const VkStridedBufferRegionKHR* pCallableShaderBindingTable, VkBuffer buffer,
+                                               VkDeviceSize offset);
     void AllocateValidationResources(const VkCommandBuffer cmd_buffer, const VkPipelineBindPoint bind_point);
     void PostCallRecordGetPhysicalDeviceProperties(VkPhysicalDevice physicalDevice,
                                                    VkPhysicalDeviceProperties* pPhysicalDeviceProperties);
     void PostCallRecordGetPhysicalDeviceProperties2(VkPhysicalDevice physicalDevice,
                                                     VkPhysicalDeviceProperties2* pPhysicalDeviceProperties2);
-    VkResult InitializeVma(VkPhysicalDevice physicalDevice, VkDevice device, VmaAllocator* pAllocator);
 };
