@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2015-2019 The Khronos Group Inc.
- * Copyright (c) 2015-2019 Valve Corporation
- * Copyright (c) 2015-2019 LunarG, Inc.
- * Copyright (c) 2015-2019 Google, Inc.
+ * Copyright (c) 2015-2020 The Khronos Group Inc.
+ * Copyright (c) 2015-2020 Valve Corporation
+ * Copyright (c) 2015-2020 LunarG, Inc.
+ * Copyright (c) 2015-2020 Google, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -30,7 +30,6 @@
 #ifdef ANDROID
 #include "vulkan_wrapper.h"
 #else
-#define NOMINMAX
 #include <vulkan/vulkan.h>
 #endif
 
@@ -56,7 +55,12 @@
 #include <functional>
 #include <limits>
 #include <memory>
+#include <string>
 #include <unordered_set>
+#include <vector>
+
+using std::string;
+using std::vector;
 
 //--------------------------------------------------------------------------------------
 // Mesh and VertexFormat Data
@@ -159,6 +163,8 @@ static const char bindStateFragUniformShaderText[] =
     "   x = vec4(bar.y);\n"
     "}\n";
 
+void SetEnvVar(const char *env_var, const char *value);
+
 // Static arrays helper
 template <class ElementT, size_t array_size>
 size_t size(ElementT (&)[array_size]) {
@@ -183,10 +189,6 @@ bool ImageFormatAndFeaturesSupported(VkPhysicalDevice phy, VkFormat format, VkIm
 bool ImageFormatAndFeaturesSupported(const VkInstance inst, const VkPhysicalDevice phy, const VkImageCreateInfo info,
                                      const VkFormatFeatureFlags features);
 
-// Validation report callback prototype
-VKAPI_ATTR VkBool32 VKAPI_CALL myDbgFunc(VkFlags msgFlags, VkDebugReportObjectTypeEXT objType, uint64_t srcObject, size_t location,
-                                         int32_t msgCode, const char *pLayerPrefix, const char *pMsg, void *pUserData);
-
 // Simple sane SamplerCreateInfo boilerplate
 VkSamplerCreateInfo SafeSaneSamplerCreateInfo();
 
@@ -203,6 +205,9 @@ bool CheckDescriptorIndexingSupportAndInitFramework(VkRenderFramework *renderFra
                                                     std::vector<const char *> &device_extension_names,
                                                     VkValidationFeaturesEXT *features, void *userData);
 
+// Helper for checking timeline semaphore support and initializing
+bool CheckTimelineSemaphoreSupportAndInitState(VkRenderFramework *renderFramework);
+
 // Dependent "false" type for the static assert, as GCC will evaluate
 // non-dependent static_asserts even for non-instantiated templates
 template <typename T>
@@ -227,6 +232,8 @@ T NearestSmaller(const T from) {
 
 class VkLayerTest : public VkRenderFramework {
   public:
+    const char *kValidationLayerName = "VK_LAYER_KHRONOS_validation";
+
     void VKTriangleTest(BsoFailSelect failCase);
 
     void GenericDrawPreparation(VkCommandBufferObj *commandBuffer, VkPipelineObj &pipelineobj, VkDescriptorSetObj &descriptorSet,
@@ -237,6 +244,7 @@ class VkLayerTest : public VkRenderFramework {
     bool AddSurfaceInstanceExtension();
     bool AddSwapchainDeviceExtension();
     VkCommandBufferObj *CommandBuffer();
+    void OOBRayTracingShadersTestBody(bool gpu_assisted);
 
   protected:
     uint32_t m_instance_api_version = 0;
@@ -248,6 +256,9 @@ class VkLayerTest : public VkRenderFramework {
     bool LoadDeviceProfileLayer(
         PFN_vkSetPhysicalDeviceFormatPropertiesEXT &fpvkSetPhysicalDeviceFormatPropertiesEXT,
         PFN_vkGetOriginalPhysicalDeviceFormatPropertiesEXT &fpvkGetOriginalPhysicalDeviceFormatPropertiesEXT);
+    bool LoadDeviceProfileLayer(
+        PFN_vkSetPhysicalDeviceFormatProperties2EXT &fpvkSetPhysicalDeviceFormatProperties2EXT,
+        PFN_vkGetOriginalPhysicalDeviceFormatProperties2EXT &fpvkGetOriginalPhysicalDeviceFormatProperties2EXT);
 
     VkLayerTest();
 };
@@ -262,14 +273,33 @@ class VkBestPracticesLayerTest : public VkLayerTest {
     void InitBestPracticesFramework();
 
   protected:
+    VkValidationFeatureEnableEXT enables_[1] = {VK_VALIDATION_FEATURE_ENABLE_BEST_PRACTICES_EXT};
+    VkValidationFeatureDisableEXT disables_[1] = {VK_VALIDATION_FEATURE_DISABLE_ALL_EXT};
+    VkValidationFeaturesEXT features_ = {VK_STRUCTURE_TYPE_VALIDATION_FEATURES_EXT, nullptr, 1, enables_, 1, disables_};
 };
 
+class VkArmBestPracticesLayerTest : public VkBestPracticesLayerTest {};
+
 class VkWsiEnabledLayerTest : public VkLayerTest {
   public:
   protected:
     VkWsiEnabledLayerTest() { m_enableWSI = true; }
 };
 
+class VkGpuAssistedLayerTest : public VkLayerTest {
+  public:
+    bool InitGpuAssistedFramework(bool request_descriptor_indexing);
+
+  protected:
+};
+
+class VkDebugPrintfTest : public VkLayerTest {
+  public:
+    void InitDebugPrintfFramework();
+
+  protected:
+};
+
 class VkBufferTest {
   public:
     enum eTestEnFlags {
@@ -422,13 +452,13 @@ struct CreatePipelineHelper {
         info_override(helper);
         helper.InitState();
 
-        for (const auto &error : errors) test.Monitor()->SetDesiredFailureMsg(flags, error);
+        for (const auto &error : errors) test.Monitor().SetDesiredFailureMsg(flags, error);
         helper.CreateGraphicsPipeline();
 
         if (positive_test) {
-            test.Monitor()->VerifyNotFound();
+            test.Monitor().VerifyNotFound();
         } else {
-            test.Monitor()->VerifyFound();
+            test.Monitor().VerifyFound();
         }
     }
 
@@ -478,13 +508,13 @@ struct CreateComputePipelineHelper {
         info_override(helper);
         helper.InitState();
 
-        for (const auto &error : errors) test.Monitor()->SetDesiredFailureMsg(flags, error);
+        for (const auto &error : errors) test.Monitor().SetDesiredFailureMsg(flags, error);
         helper.CreateComputePipeline();
 
         if (positive_test) {
-            test.Monitor()->VerifyNotFound();
+            test.Monitor().VerifyNotFound();
         } else {
-            test.Monitor()->VerifyFound();
+            test.Monitor().VerifyFound();
         }
     }
 
@@ -507,10 +537,12 @@ struct CreateNVRayTracingPipelineHelper {
     VkPipelineLayoutCreateInfo pipeline_layout_ci_ = {};
     VkPipelineLayoutObj pipeline_layout_;
     VkRayTracingPipelineCreateInfoNV rp_ci_ = {};
+    VkRayTracingPipelineCreateInfoKHR rp_ci_KHR_ = {};
     VkPipelineCacheCreateInfo pc_ci_ = {};
     VkPipeline pipeline_ = VK_NULL_HANDLE;
     VkPipelineCache pipeline_cache_ = VK_NULL_HANDLE;
     std::vector<VkRayTracingShaderGroupCreateInfoNV> groups_;
+    std::vector<VkRayTracingShaderGroupCreateInfoKHR> groups_KHR_;
     std::unique_ptr<VkShaderObj> rgs_;
     std::unique_ptr<VkShaderObj> chs_;
     std::unique_ptr<VkShaderObj> mis_;
@@ -521,50 +553,50 @@ struct CreateNVRayTracingPipelineHelper {
     static bool InitInstanceExtensions(VkLayerTest &test, std::vector<const char *> &instance_extension_names);
     static bool InitDeviceExtensions(VkLayerTest &test, std::vector<const char *> &device_extension_names);
     void InitShaderGroups();
+    void InitShaderGroupsKHR();
     void InitDescriptorSetInfo();
     void InitPipelineLayoutInfo();
     void InitShaderInfo();
     void InitNVRayTracingPipelineInfo();
+    void InitKHRRayTracingPipelineInfo();
     void InitPipelineCacheInfo();
-    void InitInfo();
+    void InitInfo(bool isKHR = false);
     void InitState();
-    void LateBindPipelineInfo();
+    void LateBindPipelineInfo(bool isKHR = false);
     VkResult CreateNVRayTracingPipeline(bool implicit_destroy = true, bool do_late_bind = true);
-
+    VkResult CreateKHRRayTracingPipeline(bool implicit_destroy = true, bool do_late_bind = true);
     // Helper function to create a simple test case (positive or negative)
     //
     // info_override can be any callable that takes a CreateNVRayTracingPipelineHelper &
     // flags, error can be any args accepted by "SetDesiredFailure".
     template <typename Test, typename OverrideFunc, typename Error>
     static void OneshotTest(Test &test, const OverrideFunc &info_override, const std::vector<Error> &errors,
-                            const VkFlags flags = VK_DEBUG_REPORT_ERROR_BIT_EXT) {
+                            const VkFlags flags = kErrorBit) {
         CreateNVRayTracingPipelineHelper helper(test);
         helper.InitInfo();
         info_override(helper);
         helper.InitState();
 
-        for (const auto &error : errors) test.Monitor()->SetDesiredFailureMsg(flags, error);
+        for (const auto &error : errors) test.Monitor().SetDesiredFailureMsg(flags, error);
         helper.CreateNVRayTracingPipeline();
-        test.Monitor()->VerifyFound();
+        test.Monitor().VerifyFound();
     }
 
     template <typename Test, typename OverrideFunc, typename Error>
-    static void OneshotTest(Test &test, const OverrideFunc &info_override, Error error,
-                            const VkFlags flags = VK_DEBUG_REPORT_ERROR_BIT_EXT) {
+    static void OneshotTest(Test &test, const OverrideFunc &info_override, Error error, const VkFlags flags = kErrorBit) {
         OneshotTest(test, info_override, std::vector<Error>(1, error), flags);
     }
 
     template <typename Test, typename OverrideFunc>
-    static void OneshotPositiveTest(Test &test, const OverrideFunc &info_override,
-                                    const VkDebugReportFlagsEXT message_flag_mask = VK_DEBUG_REPORT_ERROR_BIT_EXT) {
+    static void OneshotPositiveTest(Test &test, const OverrideFunc &info_override, const VkFlags message_flag_mask = kErrorBit) {
         CreateNVRayTracingPipelineHelper helper(test);
         helper.InitInfo();
         info_override(helper);
         helper.InitState();
 
-        test.Monitor()->ExpectSuccess(message_flag_mask);
+        test.Monitor().ExpectSuccess(message_flag_mask);
         ASSERT_VK_SUCCESS(helper.CreateNVRayTracingPipeline());
-        test.Monitor()->VerifyNotFound();
+        test.Monitor().VerifyNotFound();
     }
 };
 
@@ -609,6 +641,9 @@ VkPhysicalDevicePushDescriptorPropertiesKHR GetPushDescriptorProperties(VkInstan
 // Subgroup properties helper
 VkPhysicalDeviceSubgroupProperties GetSubgroupProperties(VkInstance instance, VkPhysicalDevice gpu);
 
+// Descriptor Indexing properties helper
+VkPhysicalDeviceDescriptorIndexingProperties GetDescriptorIndexingProperties(VkInstance instance, VkPhysicalDevice gpu);
+
 class BarrierQueueFamilyTestHelper {
   public:
     struct QueueFamilyObjs {
@@ -711,6 +746,9 @@ bool FindUnsupportedImage(VkPhysicalDevice gpu, VkImageCreateInfo &image_ci);
 VkFormat FindFormatWithoutFeatures(VkPhysicalDevice gpu, VkImageTiling tiling,
                                    VkFormatFeatureFlags undesired_features = UINT32_MAX);
 
+void AllocateDisjointMemory(VkDeviceObj *device, PFN_vkGetImageMemoryRequirements2KHR fp, VkImage mp_image,
+                            VkDeviceMemory *mp_image_mem, VkImageAspectFlagBits plane);
+
 void NegHeightViewportTests(VkDeviceObj *m_device, VkCommandBufferObj *m_commandBuffer, ErrorMonitor *m_errorMonitor);
 
 void CreateSamplerTest(VkLayerTest &test, const VkSamplerCreateInfo *pCreateInfo, std::string code = "");
@@ -723,5 +761,14 @@ void CreateBufferViewTest(VkLayerTest &test, const VkBufferViewCreateInfo *pCrea
 
 void CreateImageViewTest(VkLayerTest &test, const VkImageViewCreateInfo *pCreateInfo, std::string code = "");
 
+bool InitFrameworkForRayTracingTest(VkRenderFramework *renderFramework, bool isKHR,
+                                    std::vector<const char *> &instance_extension_names,
+                                    std::vector<const char *> &device_extension_names, void *user_data,
+                                    bool need_gpu_validation = false, bool need_push_descriptors = false,
+                                    bool deferred_state_init = false);
+
+void GetSimpleGeometryForAccelerationStructureTests(const VkDeviceObj &device, VkBufferObj *vbo, VkBufferObj *ibo,
+                                                    VkGeometryNV *geometry);
+
 void print_android(const char *c);
 #endif  // VKLAYERTEST_H
