@@ -1,7 +1,7 @@
 /*
- * Copyright (c) 2015-2019 The Khronos Group Inc.
- * Copyright (c) 2015-2019 Valve Corporation
- * Copyright (c) 2015-2019 LunarG, Inc.
+ * Copyright (c) 2015-2020 The Khronos Group Inc.
+ * Copyright (c) 2015-2020 Valve Corporation
+ * Copyright (c) 2015-2020 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -45,8 +45,6 @@ class VkImageObj;
 #include <vector>
 #include <unordered_set>
 
-using namespace std;
-
 using vk_testing::MakeVkHandles;
 
 template <class Dst, class Src>
@@ -119,19 +117,18 @@ class ErrorMonitor {
     // function and its definition.
     void SetUnexpectedError(const char *const msg);
 
+    // Set an error that should not cause a test failure
+    void SetAllowedFailureMsg(const char *const msg);
+
     VkBool32 CheckForDesiredMsg(const char *const msgString);
-    vector<string> GetOtherFailureMsgs() const;
-    VkDebugReportFlagsEXT GetMessageFlags() const;
-    bool AnyDesiredMsgFound() const;
-    bool AllDesiredMsgsFound() const;
+    VkDebugReportFlagsEXT GetMessageFlags();
     void SetError(const char *const errorString);
     void SetBailout(bool *bailout);
-    void DumpFailureMsgs() const;
 
     // Helpers
 
     // ExpectSuccess now takes an optional argument allowing a custom combination of debug flags
-    void ExpectSuccess(VkDebugReportFlagsEXT const message_flag_mask = VK_DEBUG_REPORT_ERROR_BIT_EXT);
+    void ExpectSuccess(VkDebugReportFlagsEXT const message_flag_mask = kErrorBit);
 
     void VerifyFound();
     void VerifyNotFound();
@@ -140,31 +137,108 @@ class ErrorMonitor {
     // TODO: This is stopgap to block new unexpected errors from being introduced. The long-term goal is to remove the use of this
     // function and its definition.
     bool IgnoreMessage(std::string const &msg) const;
+    std::vector<std::string> GetOtherFailureMsgs() const;
+    bool AnyDesiredMsgFound() const;
+    bool AllDesiredMsgsFound() const;
+    void DumpFailureMsgs() const;
+    void MonitorReset();
 
     VkFlags message_flags_;
     std::unordered_multiset<std::string> desired_message_strings_;
     std::unordered_multiset<std::string> failure_message_strings_;
     std::vector<std::string> ignore_message_strings_;
-    vector<string> other_messages_;
+    std::vector<std::string> allowed_message_strings_;
+    std::vector<std::string> other_messages_;
     test_platform_thread_mutex mutex_;
     bool *bailout_;
     bool message_found_;
 };
 
+struct DebugReporter {
+    void Create(VkInstance instance) NOEXCEPT;
+    void Destroy(VkInstance instance) NOEXCEPT;
+
+    ErrorMonitor error_monitor_;
+
+#ifdef VK_USE_PLATFORM_ANDROID_KHR
+    static VKAPI_ATTR VkBool32 VKAPI_CALL DebugCallback(VkDebugReportFlagsEXT message_flags, VkDebugReportObjectTypeEXT, uint64_t,
+                                                        size_t, int32_t, const char *, const char *msg, void *user_data);
+
+    const char *debug_extension_name = VK_EXT_DEBUG_REPORT_EXTENSION_NAME;
+    VkDebugReportCallbackCreateInfoEXT debug_create_info_ = {
+        VK_STRUCTURE_TYPE_DEBUG_REPORT_CALLBACK_CREATE_INFO_EXT, nullptr,
+        VK_DEBUG_REPORT_ERROR_BIT_EXT | VK_DEBUG_REPORT_WARNING_BIT_EXT | VK_DEBUG_REPORT_PERFORMANCE_WARNING_BIT_EXT,
+        &DebugCallback, &error_monitor_};
+    using DebugCreateFnType = PFN_vkCreateDebugReportCallbackEXT;
+    const char *debug_create_fn_name_ = "vkCreateDebugReportCallbackEXT";
+    using DebugDestroyFnType = PFN_vkDestroyDebugReportCallbackEXT;
+    const char *debug_destroy_fn_name_ = "vkDestroyDebugReportCallbackEXT";
+    VkDebugReportCallbackEXT debug_obj_ = VK_NULL_HANDLE;
+#else
+    static VKAPI_ATTR VkBool32 VKAPI_CALL DebugCallback(VkDebugUtilsMessageSeverityFlagBitsEXT message_severity,
+                                                        VkDebugUtilsMessageTypeFlagsEXT message_types,
+                                                        const VkDebugUtilsMessengerCallbackDataEXT *callback_data, void *user_data);
+
+    const char *debug_extension_name = VK_EXT_DEBUG_UTILS_EXTENSION_NAME;
+    VkDebugUtilsMessengerCreateInfoEXT debug_create_info_ = {
+        VK_STRUCTURE_TYPE_DEBUG_UTILS_MESSENGER_CREATE_INFO_EXT,
+        nullptr,
+        0,
+        VK_DEBUG_UTILS_MESSAGE_SEVERITY_ERROR_BIT_EXT | VK_DEBUG_UTILS_MESSAGE_SEVERITY_WARNING_BIT_EXT |
+            VK_DEBUG_UTILS_MESSAGE_SEVERITY_INFO_BIT_EXT,
+        VK_DEBUG_UTILS_MESSAGE_TYPE_VALIDATION_BIT_EXT | VK_DEBUG_UTILS_MESSAGE_TYPE_PERFORMANCE_BIT_EXT,
+        &DebugCallback,
+        &error_monitor_};
+    using DebugCreateFnType = PFN_vkCreateDebugUtilsMessengerEXT;
+    const char *debug_create_fn_name_ = "vkCreateDebugUtilsMessengerEXT";
+    using DebugDestroyFnType = PFN_vkDestroyDebugUtilsMessengerEXT;
+    const char *debug_destroy_fn_name_ = "vkDestroyDebugUtilsMessengerEXT";
+    VkDebugUtilsMessengerEXT debug_obj_ = VK_NULL_HANDLE;
+#endif
+};
+
 class VkCommandPoolObj;
 class VkCommandBufferObj;
 class VkDepthStencilObj;
 
+typedef enum {
+    kGalaxyS10,
+    kPixel3,
+    kPixelC,
+    kNexusPlayer,
+    kShieldTV,
+    kPixel3aXL,
+    kMockICD,
+} PlatformType;
+
+const std::unordered_map<PlatformType, std::string, std::hash<int>> vk_gpu_table = {
+    {kGalaxyS10, "Mali-G76"},
+    {kPixel3, "Adreno (TM) 630"},
+    {kPixelC, "NVIDIA Tegra X1"},
+    {kNexusPlayer, "PowerVR Rogue G6430"},
+    {kShieldTV, "NVIDIA Tegra X1 (nvgpu)"},
+    {kPixel3aXL, "Adreno (TM) 615"},
+    {kMockICD, "Vulkan Mock Device"},
+};
+
 class VkRenderFramework : public VkTestFramework {
   public:
-    VkInstance instance() { return inst; }
+    VkInstance instance() { return instance_; }
     VkDevice device() { return m_device->device(); }
     VkDeviceObj *DeviceObj() const { return m_device; }
     VkPhysicalDevice gpu();
     VkRenderPass renderPass() { return m_renderPass; }
-    const VkRenderPassCreateInfo &RenderPassInfo() const { return renderPass_info_; };
+    const VkRenderPassCreateInfo &RenderPassInfo() const { return m_renderPass_info; };
     VkFramebuffer framebuffer() { return m_framebuffer; }
-    ErrorMonitor *Monitor();
+    ErrorMonitor &Monitor();
+    VkPhysicalDeviceProperties physDevProps();
+
+    static bool InstanceLayerSupported(const char *layer_name, uint32_t spec_version = 0, uint32_t impl_version = 0);
+    static bool InstanceExtensionSupported(const char *extension_name, uint32_t spec_version = 0);
+
+    VkInstanceCreateInfo GetInstanceCreateInfo() const;
+    void InitFramework(void * /*unused compatibility parameter*/ = NULL, void *instance_pnext = NULL);
+    void ShutdownFramework();
 
     void InitViewport(float width, float height);
     void InitViewport();
@@ -180,9 +254,8 @@ class VkRenderFramework : public VkTestFramework {
     void InitRenderTarget(VkImageView *dsBinding);
     void InitRenderTarget(uint32_t targets, VkImageView *dsBinding);
     void DestroyRenderTarget();
-    void InitFramework(PFN_vkDebugReportCallbackEXT = NULL, void *userData = NULL, void *instance_pnext = NULL);
 
-    void ShutdownFramework();
+    bool IsPlatform(PlatformType platform);
     void GetPhysicalDeviceFeatures(VkPhysicalDeviceFeatures *features);
     void GetPhysicalDeviceProperties(VkPhysicalDeviceProperties *props);
     void InitState(VkPhysicalDeviceFeatures *features = nullptr, void *create_device_pnext = nullptr,
@@ -190,32 +263,46 @@ class VkRenderFramework : public VkTestFramework {
 
     const VkRenderPassBeginInfo &renderPassBeginInfo() const { return m_renderPassBeginInfo; }
 
-    bool InstanceLayerSupported(const char *name, uint32_t specVersion = 0, uint32_t implementationVersion = 0);
     bool EnableDeviceProfileLayer();
-    bool InstanceExtensionSupported(const char *name, uint32_t specVersion = 0);
     bool InstanceExtensionEnabled(const char *name);
-    bool DeviceExtensionSupported(VkPhysicalDevice dev, const char *layer, const char *name, uint32_t specVersion = 0);
+    bool DeviceExtensionSupported(const char *extension_name, uint32_t spec_version = 0) const;
+    bool DeviceExtensionSupported(VkPhysicalDevice, const char *, const char *name,
+                                  uint32_t spec_version = 0) const {  // deprecated
+        return DeviceExtensionSupported(name, spec_version);
+    }
     bool DeviceExtensionEnabled(const char *name);
-    bool DeviceIsMockICD();
     bool DeviceSimulation();
 
   protected:
     VkRenderFramework();
     virtual ~VkRenderFramework() = 0;
 
-    VkApplicationInfo app_info;
-    VkInstance inst;
-    VkPhysicalDevice objs[16];
-    uint32_t gpu_count;
+    DebugReporter debug_reporter_;
+    ErrorMonitor *m_errorMonitor = &debug_reporter_.error_monitor_;  // compatibility alias name
+
+    VkApplicationInfo app_info_;
+    std::vector<const char *> instance_layers_;
+    std::vector<const char *> instance_extensions_;
+    std::vector<const char *> &m_instance_extension_names = instance_extensions_;  // compatibility alias name
+    VkInstance instance_;
+    VkPhysicalDevice gpu_;
+    VkPhysicalDeviceProperties physDevProps_;
+
     VkDeviceObj *m_device;
     VkCommandPoolObj *m_commandPool;
     VkCommandBufferObj *m_commandBuffer;
     VkRenderPass m_renderPass;
-    VkRenderPassCreateInfo renderPass_info_ = {};
+    VkRenderPassCreateInfo m_renderPass_info = {};
+    std::vector<VkAttachmentDescription> m_renderPass_attachments;
+    std::vector<VkSubpassDescription> m_renderPass_subpasses;
+    std::vector<VkSubpassDependency> m_renderPass_dependencies;
+
     VkFramebuffer m_framebuffer;
+    VkFramebufferCreateInfo m_framebuffer_info;
+    std::vector<VkImageView> m_framebuffer_attachments;
+
     VkSurfaceKHR m_surface;
     VkSwapchainKHR m_swapchain;
-    ErrorMonitor *m_errorMonitor = {};
     std::vector<VkViewport> m_viewports;
     std::vector<VkRect2D> m_scissors;
     float m_lineWidth;
@@ -231,7 +318,7 @@ class VkRenderFramework : public VkTestFramework {
     bool m_addRenderPassSelfDependency;
     std::vector<VkClearValue> m_renderPassClearValues;
     VkRenderPassBeginInfo m_renderPassBeginInfo;
-    vector<std::unique_ptr<VkImageObj>> m_renderTargets;
+    std::vector<std::unique_ptr<VkImageObj>> m_renderTargets;
     float m_width, m_height;
     VkFormat m_render_target_fmt;
     VkFormat m_depth_stencil_fmt;
@@ -240,21 +327,13 @@ class VkRenderFramework : public VkTestFramework {
     float m_depth_clear_color;
     uint32_t m_stencil_clear_color;
     VkDepthStencilObj *m_depthStencil;
-    PFN_vkCreateDebugReportCallbackEXT m_CreateDebugReportCallback;
-    PFN_vkDestroyDebugReportCallbackEXT m_DestroyDebugReportCallback;
-    PFN_vkDebugReportMessageEXT m_DebugReportMessage;
-    VkDebugReportCallbackEXT m_globalMsgCallback;
-    VkDebugReportCallbackEXT m_devMsgCallback;
 
-    std::vector<const char *> m_instance_layer_names;
-    std::vector<const char *> m_instance_extension_names;
     std::vector<const char *> m_device_extension_names;
 };
 
 class VkDescriptorSetObj;
 class VkConstantBufferObj;
 class VkPipelineObj;
-class VkDescriptorSetObj;
 typedef vk_testing::Fence VkFenceObj;
 typedef vk_testing::Buffer VkBufferObj;
 typedef vk_testing::AccelerationStructure VkAccelerationStructureObj;
@@ -272,9 +351,9 @@ class VkCommandBufferObj : public vk_testing::CommandBuffer {
                          uint32_t memoryBarrierCount, const VkMemoryBarrier *pMemoryBarriers, uint32_t bufferMemoryBarrierCount,
                          const VkBufferMemoryBarrier *pBufferMemoryBarriers, uint32_t imageMemoryBarrierCount,
                          const VkImageMemoryBarrier *pImageMemoryBarriers);
-    void ClearAllBuffers(const vector<std::unique_ptr<VkImageObj>> &color_objs, VkClearColorValue clear_color,
+    void ClearAllBuffers(const std::vector<std::unique_ptr<VkImageObj>> &color_objs, VkClearColorValue clear_color,
                          VkDepthStencilObj *depth_stencil_obj, float depth_clear_value, uint32_t stencil_clear_value);
-    void PrepareAttachments(const vector<std::unique_ptr<VkImageObj>> &color_atts, VkDepthStencilObj *depth_stencil_att);
+    void PrepareAttachments(const std::vector<std::unique_ptr<VkImageObj>> &color_atts, VkDepthStencilObj *depth_stencil_att);
     void BindDescriptorSet(VkDescriptorSetObj &descriptorSet);
     void BindIndexBuffer(VkBufferObj *indexBuffer, VkDeviceSize offset, VkIndexType indexType);
     void BindVertexBuffer(VkConstantBufferObj *vertexBuffer, VkDeviceSize offset, uint32_t binding);
@@ -320,7 +399,7 @@ class VkConstantBufferObj : public VkBufferObj {
 
 class VkRenderpassObj {
   public:
-    VkRenderpassObj(VkDeviceObj *device);
+    VkRenderpassObj(VkDeviceObj *device, VkFormat format = VK_FORMAT_B8G8R8A8_UNORM);
     ~VkRenderpassObj() NOEXCEPT;
     VkRenderPass handle() { return m_renderpass; }
 
@@ -472,8 +551,8 @@ class VkDescriptorSetObj : public vk_testing::DescriptorPool {
     std::map<VkDescriptorType, int> m_type_counts;
     int m_nextSlot;
 
-    vector<VkDescriptorImageInfo> m_imageSamplerDescriptors;
-    vector<VkWriteDescriptorSet> m_writes;
+    std::vector<VkDescriptorImageInfo> m_imageSamplerDescriptors;
+    std::vector<VkWriteDescriptorSet> m_writes;
 
     vk_testing::DescriptorSetLayout m_layout;
     vk_testing::PipelineLayout m_pipeline_layout;
@@ -532,8 +611,8 @@ class VkPipelineObj : public vk_testing::Pipeline {
     void SetInputAssembly(const VkPipelineInputAssemblyStateCreateInfo *ia_state);
     void SetRasterization(const VkPipelineRasterizationStateCreateInfo *rs_state);
     void SetTessellation(const VkPipelineTessellationStateCreateInfo *te_state);
-    void SetViewport(const vector<VkViewport> viewports);
-    void SetScissor(const vector<VkRect2D> scissors);
+    void SetViewport(const std::vector<VkViewport> viewports);
+    void SetScissor(const std::vector<VkRect2D> scissors);
     void SetLineState(const VkPipelineRasterizationLineStateCreateInfoEXT *line_state);
 
     void InitGraphicsPipelineCreateInfo(VkGraphicsPipelineCreateInfo *gp_ci);
@@ -551,12 +630,12 @@ class VkPipelineObj : public vk_testing::Pipeline {
     VkPipelineTessellationStateCreateInfo const *m_te_state;
     VkPipelineDynamicStateCreateInfo m_pd_state;
     VkPipelineRasterizationLineStateCreateInfoEXT m_line_state;
-    vector<VkDynamicState> m_dynamic_state_enables;
-    vector<VkViewport> m_viewports;
-    vector<VkRect2D> m_scissors;
+    std::vector<VkDynamicState> m_dynamic_state_enables;
+    std::vector<VkViewport> m_viewports;
+    std::vector<VkRect2D> m_scissors;
     VkDeviceObj *m_device;
-    vector<VkPipelineShaderStageCreateInfo> m_shaderStages;
-    vector<VkPipelineColorBlendAttachmentState> m_colorAttachments;
+    std::vector<VkPipelineShaderStageCreateInfo> m_shaderStages;
+    std::vector<VkPipelineColorBlendAttachmentState> m_colorAttachments;
 };
 
 #endif  // VKRENDERFRAMEWORK_H
