@@ -1,7 +1,7 @@
 /*
- * Copyright (c) 2015-2019 The Khronos Group Inc.
- * Copyright (c) 2015-2019 Valve Corporation
- * Copyright (c) 2015-2019 LunarG, Inc.
+ * Copyright (c) 2015-2020 The Khronos Group Inc.
+ * Copyright (c) 2015-2020 Valve Corporation
+ * Copyright (c) 2015-2020 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -40,7 +40,7 @@ namespace {
     } while (0)
 
 #define NON_DISPATCHABLE_HANDLE_DTOR(cls, destroy_func)            \
-    cls::~cls() {                                                  \
+    cls::~cls() NOEXCEPT {                                         \
         if (initialized()) destroy_func(device(), handle(), NULL); \
     }
 
@@ -104,20 +104,21 @@ VkPhysicalDeviceFeatures PhysicalDevice::features() const {
  */
 std::vector<VkLayerProperties> GetGlobalLayers() {
     VkResult err;
-    std::vector<VkLayerProperties> layers;
     uint32_t layer_count;
+    std::vector<VkLayerProperties> layers;
 
     do {
-        layer_count = 0;
-        err = vk::EnumerateInstanceLayerProperties(&layer_count, NULL);
+        err = vk::EnumerateInstanceLayerProperties(&layer_count, nullptr);
+        assert(!err);
+        if (err || 0 == layer_count) return {};
 
-        if (err == VK_SUCCESS) {
-            layers.reserve(layer_count);
-            err = vk::EnumerateInstanceLayerProperties(&layer_count, layers.data());
-        }
-    } while (err == VK_INCOMPLETE);
+        layers.resize(layer_count);
+        err = vk::EnumerateInstanceLayerProperties(&layer_count, layers.data());
+    } while (VK_INCOMPLETE == err);
 
-    assert(err == VK_SUCCESS);
+    assert(!err);
+    if (err) return {};
+    layers.resize(layer_count);
 
     return layers;
 }
@@ -125,7 +126,7 @@ std::vector<VkLayerProperties> GetGlobalLayers() {
 /*
  * Return list of Global extensions provided by the ICD / Loader
  */
-std::vector<VkExtensionProperties> GetGlobalExtensions() { return GetGlobalExtensions(NULL); }
+std::vector<VkExtensionProperties> GetGlobalExtensions() { return GetGlobalExtensions(nullptr); }
 
 /*
  * Return list of Global extensions provided by the specified layer
@@ -133,51 +134,46 @@ std::vector<VkExtensionProperties> GetGlobalExtensions() { return GetGlobalExten
  * ICDs
  */
 std::vector<VkExtensionProperties> GetGlobalExtensions(const char *pLayerName) {
-    std::vector<VkExtensionProperties> exts;
-    uint32_t ext_count;
     VkResult err;
+    uint32_t extension_count;
+    std::vector<VkExtensionProperties> extensions;
 
     do {
-        ext_count = 0;
-        err = vk::EnumerateInstanceExtensionProperties(pLayerName, &ext_count, NULL);
+        err = vk::EnumerateInstanceExtensionProperties(nullptr, &extension_count, nullptr);
+        assert(!err);
+        if (err || 0 == extension_count) return {};
 
-        if (err == VK_SUCCESS) {
-            exts.resize(ext_count);
-            err = vk::EnumerateInstanceExtensionProperties(pLayerName, &ext_count, exts.data());
-        }
-    } while (err == VK_INCOMPLETE);
+        extensions.resize(extension_count);
+        err = vk::EnumerateInstanceExtensionProperties(nullptr, &extension_count, extensions.data());
+    } while (VK_INCOMPLETE == err);
 
-    assert(err == VK_SUCCESS);
+    assert(!err);
+    if (err) return {};
+    extensions.resize(extension_count);
 
-    return exts;
+    return extensions;
 }
 
-/*
- * Return list of PhysicalDevice extensions provided by the ICD / Loader
- */
-std::vector<VkExtensionProperties> PhysicalDevice::extensions() const { return extensions(NULL); }
-
 /*
  * Return list of PhysicalDevice extensions provided by the specified layer
  * If pLayerName is NULL, will return extensions for ICD / loader.
  */
 std::vector<VkExtensionProperties> PhysicalDevice::extensions(const char *pLayerName) const {
-    std::vector<VkExtensionProperties> exts;
     VkResult err;
-
+    uint32_t extension_count;
+    std::vector<VkExtensionProperties> extensions;
     do {
-        uint32_t extCount = 0;
-        err = vk::EnumerateDeviceExtensionProperties(handle(), pLayerName, &extCount, NULL);
+        err = vk::EnumerateDeviceExtensionProperties(handle(), pLayerName, &extension_count, nullptr);
+        if (err || 0 == extension_count) return {};
 
-        if (err == VK_SUCCESS) {
-            exts.resize(extCount);
-            err = vk::EnumerateDeviceExtensionProperties(handle(), pLayerName, &extCount, exts.data());
-        }
-    } while (err == VK_INCOMPLETE);
+        extensions.resize(extension_count);
+        err = vk::EnumerateDeviceExtensionProperties(handle(), pLayerName, &extension_count, extensions.data());
+    } while (VK_INCOMPLETE == err);
 
-    assert(err == VK_SUCCESS);
+    if (err) return {};
+    extensions.resize(extension_count);
 
-    return exts;
+    return extensions;
 }
 
 bool PhysicalDevice::set_memory_type(const uint32_t type_bits, VkMemoryAllocateInfo *info, const VkFlags properties,
@@ -203,22 +199,21 @@ bool PhysicalDevice::set_memory_type(const uint32_t type_bits, VkMemoryAllocateI
  * Return list of PhysicalDevice layers
  */
 std::vector<VkLayerProperties> PhysicalDevice::layers() const {
-    std::vector<VkLayerProperties> layer_props;
     VkResult err;
-
+    uint32_t layer_count;
+    std::vector<VkLayerProperties> layers;
     do {
-        uint32_t layer_count = 0;
-        err = vk::EnumerateDeviceLayerProperties(handle(), &layer_count, NULL);
+        err = vk::EnumerateDeviceLayerProperties(handle(), &layer_count, nullptr);
+        if (err || 0 == layer_count) return {};
 
-        if (err == VK_SUCCESS) {
-            layer_props.reserve(layer_count);
-            err = vk::EnumerateDeviceLayerProperties(handle(), &layer_count, layer_props.data());
-        }
-    } while (err == VK_INCOMPLETE);
+        layers.resize(layer_count);
+        err = vk::EnumerateDeviceLayerProperties(handle(), &layer_count, layers.data());
+    } while (VK_INCOMPLETE == err);
 
-    assert(err == VK_SUCCESS);
+    if (err) return {};
+    layers.resize(layer_count);
 
-    return layer_props;
+    return layers;
 }
 
 QueueCreateInfoArray::QueueCreateInfoArray(const std::vector<VkQueueFamilyProperties> &queue_props)
@@ -239,7 +234,7 @@ QueueCreateInfoArray::QueueCreateInfoArray(const std::vector<VkQueueFamilyProper
     }
 }
 
-Device::~Device() {
+Device::~Device() NOEXCEPT {
     if (!initialized()) return;
 
     vk::DestroyDevice(handle(), NULL);
@@ -348,7 +343,8 @@ const Device::QueueFamilyQueues &Device::queue_family_queues(uint32_t queue_fami
 }
 
 void Device::init_formats() {
-    for (int f = VK_FORMAT_BEGIN_RANGE; f <= VK_FORMAT_END_RANGE; f++) {
+    // For each 1.0 core format, undefined = first, 12x12_SRGB_BLOCK = last
+    for (int f = VK_FORMAT_UNDEFINED; f <= VK_FORMAT_ASTC_12x12_SRGB_BLOCK; f++) {
         const VkFormat fmt = static_cast<VkFormat>(f);
         const VkFormatProperties props = format_properties(fmt);
 
@@ -426,7 +422,7 @@ VkResult Queue::wait() {
     return result;
 }
 
-DeviceMemory::~DeviceMemory() {
+DeviceMemory::~DeviceMemory() NOEXCEPT {
     if (initialized()) vk::FreeMemory(device(), handle(), NULL);
 }
 
@@ -469,9 +465,9 @@ NON_DISPATCHABLE_HANDLE_DTOR(Fence, vk::DestroyFence)
 
 void Fence::init(const Device &dev, const VkFenceCreateInfo &info) { NON_DISPATCHABLE_HANDLE_INIT(vk::CreateFence, dev, &info); }
 
-VkResult Fence::wait(VkBool32 wait_all, uint64_t timeout) const {
+VkResult Fence::wait(uint64_t timeout) const {
     VkFence fence = handle();
-    return vk::WaitForFences(device(), 1, &fence, wait_all, timeout);
+    return vk::WaitForFences(device(), 1, &fence, VK_TRUE, timeout);
 }
 
 NON_DISPATCHABLE_HANDLE_DTOR(Semaphore, vk::DestroySemaphore)
@@ -624,7 +620,6 @@ VkMemoryRequirements2 AccelerationStructure::memory_requirements() const {
     memoryRequirementsInfo.sType = VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_MEMORY_REQUIREMENTS_INFO_NV;
     memoryRequirementsInfo.type = VK_ACCELERATION_STRUCTURE_MEMORY_REQUIREMENTS_TYPE_OBJECT_NV;
     memoryRequirementsInfo.accelerationStructure = handle();
-
     VkMemoryRequirements2 memoryRequirements = {};
     vkGetAccelerationStructureMemoryRequirementsNV(device(), &memoryRequirementsInfo, &memoryRequirements);
     return memoryRequirements;
@@ -676,14 +671,19 @@ void AccelerationStructure::init(const Device &dev, const VkAccelerationStructur
     }
 }
 
-void AccelerationStructure::create_scratch_buffer(const Device &dev, Buffer *buffer) {
+void AccelerationStructure::create_scratch_buffer(const Device &dev, Buffer *buffer, VkBufferCreateInfo *pCreateInfo) {
     VkMemoryRequirements scratch_buffer_memory_requirements = build_scratch_memory_requirements().memoryRequirements;
 
     VkBufferCreateInfo create_info = {};
-    create_info.sType = VK_STRUCTURE_TYPE_BUFFER_CREATE_INFO;
     create_info.size = scratch_buffer_memory_requirements.size;
-    create_info.usage = VK_BUFFER_USAGE_RAY_TRACING_BIT_NV;
-    return buffer->init(dev, create_info, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT);
+    if (pCreateInfo) {
+        create_info.sType = pCreateInfo->sType;
+        create_info.usage = pCreateInfo->usage;
+    } else {
+        create_info.sType = VK_STRUCTURE_TYPE_BUFFER_CREATE_INFO;
+        create_info.usage = VK_BUFFER_USAGE_RAY_TRACING_BIT_NV;
+    }
+    buffer->init(dev, create_info, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT);
 }
 
 NON_DISPATCHABLE_HANDLE_DTOR(ShaderModule, vk::DestroyShaderModule)
@@ -811,7 +811,7 @@ DescriptorSet *DescriptorPool::alloc_sets(const Device &dev, const DescriptorSet
     return (set.empty()) ? NULL : set[0];
 }
 
-DescriptorSet::~DescriptorSet() {
+DescriptorSet::~DescriptorSet() NOEXCEPT {
     if (initialized()) {
         // Only call vk::Free* on sets allocated from pool with usage *_DYNAMIC
         if (containing_pool_->getDynamicUsage()) {
@@ -827,7 +827,7 @@ void CommandPool::init(const Device &dev, const VkCommandPoolCreateInfo &info) {
     NON_DISPATCHABLE_HANDLE_INIT(vk::CreateCommandPool, dev, &info);
 }
 
-CommandBuffer::~CommandBuffer() {
+CommandBuffer::~CommandBuffer() NOEXCEPT {
     if (initialized()) {
         VkCommandBuffer cmds[] = {handle()};
         vk::FreeCommandBuffers(dev_handle_, cmd_pool_, 1, cmds);
